﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	/// <summary>
	/// Generátor kostek vrací náhodné kostky podle vzorů ze souboru
	/// </summary>
	public class GeneratorKostek
    {
        /// <summary>
        /// Seznam vzorů kostek
        /// </summary>
        private List<int[,]> vzory;
        /// <summary>
        /// Instance generátoru náhodných čísel
        /// </summary>
        private Random nahodnyGenerator;
        /// <summary>
        /// Cesta k souboru se vzory
        /// </summary>
        private string soubor;

        /// <summary>
        /// Vytvoří nový generátor kostek
        /// </summary>
        /// <param name="soubor">Soubor se vzory kostek</param>
        /// <param name="pocet">Počet kostek od začátku souboru</param>
        public GeneratorKostek(string soubor)
        {
            vzory = new List<int[,]>();
            nahodnyGenerator = new Random();
            this.soubor = soubor;
        }

        /// <summary>
        /// Načte kostky z externího souboru
        /// </summary>
        /// <param name="soubor">Cesta k souboru</param>
        public void NactiKostky()
        {
            using (StreamReader sr = new StreamReader(soubor))
            {
                // příprava proměnných
                string radka;
                int[,] vzor = new int[4, 4];
                int j = 0;
                int cisloRadky = 1;
                // přečtení všech řádek textového souboru
                while ((radka = sr.ReadLine()) != null)
                {
                    // každou pátou řádku vynecháme, jinak pokračujeme
                    if ((cisloRadky % 5) != 0)
                    {
                        // načtení čísel z řádky do vzoru
                        for (int i = 0; i < 4; i++)
                            vzor[i, j] = int.Parse(radka[i].ToString());
                        j++;
                        // načetli jsme poslední řádku?
                        if (j > 3)
                        {
                            // vložíme vzor
                            vzory.Add(vzor);
                            // a resetujeme proměnné
                            j = 0;
                            vzor = new int[4, 4];
                        }
                    }
                    cisloRadky++;
                }
            }
        }

        /// <summary>
        /// Vygeneruje a vrátí novou kostku
        /// </summary>
        /// <param name="pocet">Počet kostek od začátku kolekce ke generování</param>
        /// <returns>Nová kostka</returns>
        public Kostka Generuj(int pocet)
        {
            int index = nahodnyGenerator.Next(0, pocet);

            return new Kostka(vzory[index]);
        }


    }
}
