﻿using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	/// <summary>
	/// Herní obrazovka, umožňující definovat herní komponenty pro 1 obrazovku a mezi obrazovkami poté přepínat
	/// </summary>
	public class HerniObrazovka
    {
        /// <summary>
        /// Seznam komponent, které obrazovka používá
        /// </summary>
        private List<GameComponent> komponenty;
        /// <summary>
        /// Instance hry
        /// </summary>
        private Hra hra;

        /// <summary>
        /// Vytvoří novou herní obrazovku a přidá do ní komponenty
        /// </summary>
        /// <param name="hra">Instance hry</param>
        /// <param name="komponenty">Několik komponent, které má obrazovka obsahovat</param>
        public HerniObrazovka(Hra hra, params GameComponent[] komponenty)
        {
            this.hra = hra;
            this.komponenty = new List<GameComponent>();
            foreach (GameComponent komponenta in komponenty)
            {
                PridejKomponentu(komponenta);
            }
        }

        /// <summary>
        /// Přidá komponenty do herní obrazovky
        /// </summary>
        /// <param name="komponenty"></param>
        public void PridejKomponentu(GameComponent komponenta)
        {
            komponenty.Add(komponenta);
            if (!hra.Components.Contains(komponenta))
                hra.Components.Add(komponenta);
        }

        /// <summary>
        /// Vrátí komponenty herní obrazovky
        /// </summary>
        /// <returns>Pole komponent herní obrazovky</returns>
        public GameComponent[] VratKomponenty()
        {
            return komponenty.ToArray();
        }

    }
}
