﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System.Linq;

namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	/// <summary>
	/// Hlavní třída naší hry, stará se o přepínání mezi herními komponentami a umožňuje jim sdílet herní data
	/// </summary>
	public class Hra : Microsoft.Xna.Framework.Game
	{
		/// <summary>
		/// Manažer grafiky
		/// </summary>
		private GraphicsDeviceManager graphics;
		/// <summary>
		/// SpriteBatch
		/// </summary>
		public LepsiSpriteBatch spriteBatch;
		// velikost okna
		public int sirkaOkna = 1280, vyskaOkna = 720;
		/// <summary>
		/// Textury
		/// </summary>
		private Texture2D kostkyPozadi;
		/// <summary>
		/// Zvukové efekty
		/// </summary>
		private SoundEffect zvukRada;
		/// <summary>
		/// Hudba
		/// </summary>
		public Song hudba_zardax;
		/// <summary>
		/// Fonty
		/// </summary>
		public SpriteFont fontCourierNew, fontBlox, fontBloxMaly;
		/// <summary>
		/// Aktuální a minulý stav stiknutých kláves
		/// </summary>
		public KeyboardState klavesy, klavesyMinule;
		/// <summary>
		/// Aktuální hráč
		/// </summary>
		public Hrac hrac;
		/// <summary>
		/// Herní obrazovky
		/// </summary>
		public HerniObrazovka obrazovkaMenu, obrazovkaLevel, obrazovkaAutori, obrazovkaSkore;

		/// <summary>
		/// Vytvoří novou instanci hry
		/// </summary>
		public Hra()
		{
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
		}

		/// <summary>
		/// Inicializace hry
		/// </summary>
		protected override void Initialize()
		{
			// vytvoření komponent
			KomponentaMraky mraky = new KomponentaMraky(this);
			KomponentaLevel level = new KomponentaLevel(this);
			KomponentaAutori autori = new KomponentaAutori(this);
			KomponentaSkoreTabulka skoreTabulka = new KomponentaSkoreTabulka(this);
			KomponentaPolozkyMenu polozkyMenu = new KomponentaPolozkyMenu(this, new Vector2(700, 250), Color.Red, Color.Yellow, 80);
			polozkyMenu.PridejPolozku("StArT");
			polozkyMenu.PridejPolozku("SkOrE");
			polozkyMenu.PridejPolozku("AuToRi");
			polozkyMenu.PridejPolozku("KoNeC");
			KomponentaMenu menu = new KomponentaMenu(this, polozkyMenu);

			// herní obrazovky
			obrazovkaMenu = new HerniObrazovka(this, mraky, menu);
			obrazovkaLevel = new HerniObrazovka(this, mraky, level);
			obrazovkaMenu = new HerniObrazovka(this, mraky, menu, polozkyMenu);
			obrazovkaAutori = new HerniObrazovka(this, mraky, autori);
			obrazovkaSkore = new HerniObrazovka(this, mraky, skoreTabulka);

			// vypnutí komponent
			foreach (GameComponent komponenta in Components)
			{
				PrepniKomponentu(komponenta, false);
			}

			// velikost okna
			graphics.PreferredBackBufferWidth = sirkaOkna;
			graphics.PreferredBackBufferHeight = vyskaOkna;
			graphics.IsFullScreen = false;
			graphics.ApplyChanges();

			base.Initialize(); // pokračování inicializace
		}

		/// <summary>
		/// Zapne/vypne komponentu v dané herní obrazovce
		/// </summary>
		/// <param name="komponenta">Komponenta</param>
		/// <param name="zapnout">True pokud se má komponenta zapnout, false pokud se má vypnout</param>
		private void PrepniKomponentu(GameComponent komponenta, bool zapnout)
		{
			komponenta.Enabled = zapnout;
			if (komponenta is DrawableGameComponent)
				((DrawableGameComponent)komponenta).Visible = zapnout;
		}

		/// <summary>
		/// Přepne herní obrazovku
		/// </summary>
		/// <param name="obrazovka"></param>
		public void PrepniObrazovku(HerniObrazovka obrazovka)
		{
			GameComponent[] povolene = obrazovka.VratKomponenty();
			foreach (GameComponent komponenta in Components)
			{
				bool povolena = povolene.Contains(komponenta);
				PrepniKomponentu(komponenta, povolena);
			}
			klavesyMinule = klavesy;
		}

		/// <summary>
		/// Vrátí true, pokud je klávesa nově stisknutá a předtím nebyla
		/// </summary>
		/// <param name="klavesa">Klávesa</param>
		/// <returns>True, pokud je klávesa nově stisknutá</returns>
		public bool NovaKlavesa(Keys klavesa)
		{
			return klavesy.IsKeyDown(klavesa) && klavesyMinule.IsKeyUp(klavesa);
		}

		/// <summary>
		/// Načtení datového obsahu hry
		/// </summary>
		protected override void LoadContent()
		{
			// vytvoření SpriteBatche pro textury
			spriteBatch = new LepsiSpriteBatch(GraphicsDevice);
			// načtení spritů
			kostkyPozadi = Content.Load<Texture2D>(@"Sprity\pozadi_kostky");
			// načtení fontů
			fontCourierNew = Content.Load<SpriteFont>(@"Fonty\font_courier_new");
			fontBlox = Content.Load<SpriteFont>(@"Fonty\font_blox");
			fontBloxMaly = Content.Load<SpriteFont>(@"Fonty\font_blox_maly");
			// načtení zvuků
			zvukRada = Content.Load<SoundEffect>(@"Zvuky\zvuk_rada");
			hudba_zardax = Content.Load<Song>(@"Zvuky\hudba_zardax");

			// přehrávání hudby
			MediaPlayer.IsRepeating = true;
			PrepniHudbu(hudba_zardax);
			PrepniObrazovku(obrazovkaMenu);
		}

		/// <summary>
		/// Přepne hudbu v přehrávači
		/// </summary>
		/// <param name="hudba">Hudba</param>
		public void PrepniHudbu(Song hudba)
		{
			// Nehraje již ta samá hudba?
			if (MediaPlayer.Queue.ActiveSong != hudba)
				MediaPlayer.Play(hudba);
		}

		/// <summary>
		/// Uvolnění zdrojů
		/// </summary>
		protected override void UnloadContent()
		{

		}

		/// <summary>
		/// Realtime logika a volání logiky komponent
		/// </summary>
		/// <param name="gameTime">Herní čas</param>
		protected override void Update(GameTime gameTime)
		{
			// Ukončení hry na XBoxu
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			// stavy klávesnice
			klavesyMinule = klavesy;
			klavesy = Keyboard.GetState();

			// volání logiky komponent
			base.Update(gameTime);
		}

		/// <summary>
		/// Hlavní vykreslovací metoda. Vykresluje společnou grafiku a také volá vykreslení komponent
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			// vymazání plátna
			GraphicsDevice.Clear(Color.CornflowerBlue);
			// započetí vykreslování
			spriteBatch.Begin();
			// vykreslení spritů
			spriteBatch.Draw(kostkyPozadi, new Vector2(0, 0), Color.White);
			// ukončení vykreslování
			spriteBatch.End();

			// volání vykreslování komponent
			base.Draw(gameTime);
		}
	}
}
