﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.IO;

namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	/// <summary>
	/// Komponenta pro skrolující text s autory hry
	/// </summary>
	public class KomponentaAutori : Microsoft.Xna.Framework.DrawableGameComponent
    {

        /// <summary>
        /// Instance hry, obsahující závislosti
        /// </summary>
        private Hra hra;
        /// <summary>
        /// Pozadí komponenty
        /// </summary>
        private Texture2D pozadi;
        /// <summary>
        /// Text ke skrolování
        /// </summary>
        private string text;
        /// <summary>
        /// Startovní pozice textu
        /// </summary>
        private Vector2 startovniPozice;
        /// <summary>
        /// Aktuální pozice textu
        /// </summary>
        private Vector2 pozice;
        /// <summary>
        /// Výška celého bloku skrolujícího textu v pixelech
        /// </summary>
        private float vyskaTextu;
        /// <summary>
        /// Font pro vykreslení textu
        /// </summary>
        private SpriteFont font;

        /// <summary>
        /// Vytvoří novou komponentu Autori
        /// </summary>
        /// <param name="hra">Instance hry</param>
        public KomponentaAutori(Hra hra)
            : base(hra)
        {
            this.hra = hra;
        }

        /// <summary>
        /// Načte text ze souboru do proměnné
        /// </summary>
        public void NactiText()
        {
            text = "";
            using (StreamReader sr = new StreamReader("autori.txt"))
            {
                text = sr.ReadToEnd();
            }
        }

        /// <summary>
        /// Inicializace a vytvoření komponent
        /// </summary>
        public override void Initialize()
        {
            NactiText();
            startovniPozice = new Vector2(510, 550);
            pozice = startovniPozice;

            base.Initialize();
        }

        /// <summary>
        /// Načtení potřebných zdrojů
        /// </summary>
        protected override void LoadContent()
        {
            pozadi = hra.Content.Load<Texture2D>(@"Sprity\pozadi_menu");
            font = hra.Content.Load<SpriteFont>(@"Fonty\font_courier_new");
            vyskaTextu = font.MeasureString(text).Y;

            base.LoadContent();
        }

        /// <summary>
        /// Updatování stavu komponenty
        /// </summary>
        /// <param name="gameTime">Herní čas</param>
        public override void Update(GameTime gameTime)
        {
            // návrat do menu
            if (hra.NovaKlavesa(Keys.Escape))
                hra.PrepniObrazovku(hra.obrazovkaMenu);

            // posun pozice textu nahuru
            pozice.Y -= 0.3F;

            // reset pozice textu
            if (pozice.Y < startovniPozice.Y - (vyskaTextu + 320))
                pozice = startovniPozice;

            base.Update(gameTime);
        }

        /// <summary>
        /// Provede vykreslení
        /// </summary>
        /// <param name="gameTime">Herní čas</param>
        public override void Draw(GameTime gameTime)
        {
            // vykreslení pozadí
            hra.spriteBatch.Begin();
            hra.spriteBatch.Draw(pozadi, new Vector2(0, 0), Color.White);
            hra.spriteBatch.End();

            // vyklreslení skrolujícího textu pomocí clippingu (nůžek):
            // zapnutí clippingu
            RasterizerState rs = new RasterizerState();
            rs.ScissorTestEnable = true;
            hra.spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, null, null, rs);
            // uložení současných nůžek
            Rectangle currentRect = hra.spriteBatch.GraphicsDevice.ScissorRectangle;
            // nastavení nůžek
            hra.spriteBatch.GraphicsDevice.ScissorRectangle = new Rectangle(488, 240, 570, 335);
            // vykreslení textu
            hra.spriteBatch.TextSeStinem(font, text, pozice, Color.Red);
            // reset původních nůžek
            hra.spriteBatch.GraphicsDevice.ScissorRectangle = currentRect;
            hra.spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
