﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System;

namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	/// <summary>
	/// Level s běžící hrou
	/// </summary>
	public class KomponentaLevel : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /// <summary>
        /// Instance hry
        /// </summary>
	    private Hra hra;
        /// <summary>
        /// Sprite pozadí
        /// </summary>
        private Texture2D pozadi;
        // <summary>
        /// Instance právě padající kostky
        /// </summary>
        private Kostka kostka;
        /// <summary>
        /// Instance příští kostky
        /// </summary>
        private Kostka pristiKostka;
        /// <summary>
        /// Pozice hrací plochy
        /// </summary>
        private Vector2 poziceHraciPlochy;
        /// <summary>
        /// Sprite políčka
        /// </summary>
        private Texture2D[] sprityPolicek;
        /// <summary>
        /// Generátor kostek
        /// </summary>
        private GeneratorKostek generatorKostek;
        /// <summary>
        /// Instance hrací plochy
        /// </summary>
        private HraciPlocha hraciPlocha;
        /// <summary>
        /// Rychlost hry
        /// </summary>
        private float rychlost;
        /// <summary>
        /// Čas uplynulý od posledníhopádu kostku
        /// </summary>
        private float casOdPoslednihoPadu;
        /// <summary>
        /// Čas od posledního stisku klávesy
        /// </summary>
        private float casOdPoslednihoStisku;
        /// <summary>
        /// Prodleva klávesy
        /// </summary>
        private float prodlevaKlavesy;
        /// <summary>
        /// Font
        /// </summary>
        private SpriteFont font;
        /// <summary>
        /// Zvuk zmizení řady
        /// </summary>
        private SoundEffect zvukRada;
        /// <summary>
        /// Stav hry
        /// </summary>
        public enum eStavHry
        {
            Hra,
            Pauza,
        }
        public eStavHry stavHry;
        /// <summary>
        /// Sprite splash obrazovky pauzy
        /// </summary>
        private Texture2D sprPauza;
        /// <summary>
        /// Pozice zaměřovače
        /// </summary>
        private Vector2 poziceZamerovace;
        /// <summary>
        /// Zvyšuje se průhlednost zaměřovače
        /// </summary>
        private int zamerovacSmer;
        /// <summary>
        /// Průhlednost zaměřovače
        /// </summary>
        private float zamerovacAlfa;

        /// <summary>
        /// Vytvoří novou instanci komponenty a uloží instanci hry
        /// </summary>
        /// <param name="hra">Instance hry</param>
	    public KomponentaLevel(Hra hra)
		    : base(hra)
	    {
		    this.hra = hra;
	    }

        /// <summary>
        /// Inicializace komponenty
        /// </summary>
	    public override void Initialize()
	    {
            // vytvoření vzoru pro testovací kostku
            int[,] vzor = new int[4, 4];
            vzor[1, 0] = 1;
            vzor[2, 0] = 1;
            vzor[0, 1] = 1;
            vzor[1, 1] = 1;
            poziceHraciPlochy = new Vector2(366, 50);
            generatorKostek = new GeneratorKostek("netfx.dll");
            generatorKostek.NactiKostky();

            hraciPlocha = new HraciPlocha(10, 20, poziceHraciPlochy);

            pristiKostka = generatorKostek.Generuj(7);
            DalsiKostka();

            rychlost = 1;
            casOdPoslednihoPadu = 0;

            prodlevaKlavesy = 0.06f;
            casOdPoslednihoStisku = 0;

            hra.hrac = new Hrac();
            stavHry = eStavHry.Hra;
            // pulzování zaměřovače
            zamerovacAlfa = 0.0f;
            zamerovacSmer = 1;

		    base.Initialize();
	    }

        /// <summary>
        /// Načtení dat
        /// </summary>
	    protected override void LoadContent()
	    {
            pozadi = hra.Content.Load<Texture2D>(@"Sprity\pozadi_level");
            font = hra.Content.Load<SpriteFont>("Fonty/font_blox_maly");
            zvukRada = hra.Content.Load<SoundEffect>(@"Zvuky\zvuk_rada");
            sprPauza = hra.Content.Load<Texture2D>(@"Sprity\spr_pauza");
            // Načtení spritů kostek
            sprityPolicek = new Texture2D[15];
            for (int i = 0; i < 15; i++)
                sprityPolicek[i] = hra.Content.Load<Texture2D>("Sprity\\Policka\\" + (i + 1).ToString());
		    base.LoadContent();
	    }

		/// <summary>
		/// Reset hry při přepnutí na komponentu
		/// </summary>
		protected override void OnEnabledChanged(object sender, EventArgs args)
		{
			if (Enabled)
			{
				stavHry = eStavHry.Hra;
				hra.PrepniHudbu(hra.hudba_zardax);
				hra.hrac = new Hrac();
				hraciPlocha.Vyprazdni();
				rychlost = 1;
				// vygenerování a dosazení kostek
				pristiKostka = generatorKostek.Generuj(7);
				DalsiKostka();
			}
			base.OnEnabledChanged(sender, args);
		}

		/// <summary>
		/// Shodí další kostku
		/// </summary>
		public void DalsiKostka()
        {
            kostka = pristiKostka;
            kostka.pozice = hraciPlocha.PosadKostku(kostka);
            pristiKostka = generatorKostek.Generuj(7);
        }

        /// <summary>
        /// Real-time logika
        /// </summary>
        /// <param name="gameTime">Herní čas</param>
        public override void Update(GameTime gameTime)
        {
            // Hra
            if (stavHry == eStavHry.Hra)
            {
                if (hra.NovaKlavesa(Keys.Enter) || hra.NovaKlavesa(Keys.Up))
                {
                    kostka.Orotuj();
                    // Orotovaná kostka koliduje - vrátím rotaci zpět
                    if (hraciPlocha.Kolize(kostka, kostka.pozice))
                        for (int i = 0; i < 3; i++)
                            kostka.Orotuj();
                }

                // Sekundy uběhlé od posledního update
                float sekundy = (float)gameTime.ElapsedGameTime.TotalSeconds;

                // Obsluha časovaných kláves
                casOdPoslednihoStisku += sekundy;
                if (casOdPoslednihoStisku > prodlevaKlavesy)
                {
                    if ((hra.klavesy.IsKeyDown(Keys.Right)) &&
                       (!hraciPlocha.Kolize(kostka, new Vector2(kostka.pozice.X + 1, kostka.pozice.Y))))
                        kostka.pozice.X++; // pohyb doprava
                    if ((hra.klavesy.IsKeyDown(Keys.Left)) &&
                        (!hraciPlocha.Kolize(kostka, new Vector2(kostka.pozice.X - 1, kostka.pozice.Y))))
                        kostka.pozice.X--; // doleva
                    casOdPoslednihoStisku = 0;
                }

                float casMeziPady = rychlost;

                // Zrychlení pádu kostky
                if (hra.klavesy.IsKeyDown(Keys.RightControl) && (rychlost > 0.15f))
                    casMeziPady = 0.15f;

                // Uzemnění kostky - klávesa dolů
                if (hra.NovaKlavesa(Keys.Down))
                {
                    kostka.pozice = hraciPlocha.SpocitejSouradniceZamerovace(kostka);
                    casOdPoslednihoPadu += casMeziPady;
                }

                // Pád kostiček
                casOdPoslednihoPadu += sekundy;
                if (casOdPoslednihoPadu >= casMeziPady)
                {
                    kostka.Spadni();
                    // kostka koliduje, tedy dopadla
                    if (hraciPlocha.Kolize(kostka, kostka.pozice))
                    {
                        kostka.pozice.Y--;
                        hraciPlocha.Pripoj(kostka);
                        int rady = hraciPlocha.VymazRady();

                        if (rady > 0)
                            zvukRada.Play();

                        hra.hrac.rady += rady;
                        // bodování
                        switch (rady)
                        {
                            case 1: hra.hrac.body += hra.hrac.level * 40 + 40; break;
                            case 2: hra.hrac.body += hra.hrac.level * 100 + 100; break;
                            case 3: hra.hrac.body += hra.hrac.level * 300 + 300; break;
                            case 4: hra.hrac.body += hra.hrac.level * 1200 + 1200; break;
                        }
                        int level = hra.hrac.rady / 10 + 1;
                        if (hra.hrac.level != level)
                        {
                            hra.hrac.level = level;
                            rychlost = rychlost * 5 / 6;
                        }

                        DalsiKostka();

						// Prohra
						if (hraciPlocha.Kolize(kostka, kostka.pozice))
							hra.PrepniObrazovku(hra.obrazovkaSkore);
                    }

                    casOdPoslednihoPadu = 0;
                }

                // Pauza - Escape
                if (hra.klavesy.IsKeyDown(Keys.Escape))
                {
                    MediaPlayer.Pause();
                    stavHry = eStavHry.Pauza;
                }

                // Blikání zaměřovače
                zamerovacAlfa += 0.02f * zamerovacSmer;
                if (zamerovacAlfa > 0.5)
                    zamerovacSmer = -1;
                if (zamerovacAlfa < 0.2)
                    zamerovacSmer = 1;

                poziceZamerovace = hraciPlocha.SpocitejSouradniceZamerovace(kostka);

            }

            // Pauza
            if (stavHry == eStavHry.Pauza)
            {
                if (hra.klavesy.IsKeyDown(Keys.A))
                    hra.Exit(); // Ukončení

                if (hra.klavesy.IsKeyDown(Keys.N))
                {
                    stavHry = eStavHry.Hra; // Pokračování
                    MediaPlayer.Resume();
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// Vykreslující metoda
        /// </summary>
        /// <param name="spriteBatch"></param>
	    public override void Draw(GameTime gameTime)
	    {
            hra.spriteBatch.Begin();
            hra.spriteBatch.Draw(pozadi, new Vector2(0, 0), Color.White);
            // vykreslení herních objektů
            kostka.Vykresli(poziceHraciPlochy, hra.spriteBatch, sprityPolicek);
            pristiKostka.Vykresli(new Vector2(930, 200 + (40 * zamerovacAlfa)), hra.spriteBatch, sprityPolicek);
            kostka.VykresliZamerovac(poziceHraciPlochy, hra.spriteBatch, sprityPolicek, poziceZamerovace, zamerovacAlfa);
            hraciPlocha.Vykresli(hra.spriteBatch, sprityPolicek);

            // vykreslení HUDu
            hra.spriteBatch.TextSeStinem(font, "skore\n " + hra.hrac.body.ToString(), new Vector2(30, 390), Color.Red);
            hra.spriteBatch.TextSeStinem(font, "level\n " + hra.hrac.level.ToString(), new Vector2(215, 390), Color.Red);

            // Pauza splash
            if (stavHry == eStavHry.Pauza)
            {
                hra.spriteBatch.Draw(sprPauza, new Rectangle(0, 0, hra.sirkaOkna, hra.vyskaOkna), Color.White);
                hra.spriteBatch.TextSeStinem(font, "hra pozastavena", new Vector2(480, 260), Color.Red);
                hra.spriteBatch.TextSeStinem(hra.fontCourierNew, "Opravdu si přejete hru ukončit?\n\n\'A\' - ukončení hry \n\n\'N\' - pokračovat ve hře", new Vector2(440, 340), Color.Red);
            }

            hra.spriteBatch.End();

		    base.Draw(gameTime);
	    }


    }
}
