﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	public class KomponentaMenu : Microsoft.Xna.Framework.DrawableGameComponent
	{
		/// <summary>
		/// Textura pozadí
		/// </summary>
		private Texture2D pozadi;
		/// <summary>
		/// Instance hry
		/// </summary>
		private Hra hra;
		/// <summary>
		/// Komponenta obsahující položky menu
		/// </summary>
		private KomponentaPolozkyMenu polozkyMenu;

		/// <summary>
		///  Vytvoří mnovou komponentu menu
		/// </summary>
		/// <param name="hra">Instance hry</param>
		/// <param name="polozkyMenu">Komponenta pro položky menu</param>
		public KomponentaMenu(Hra hra, KomponentaPolozkyMenu polozkyMenu)
			: base(hra)
		{
			this.hra = hra;
			this.polozkyMenu = polozkyMenu;
		}

		/// <summary>
		/// Inicializace komponenty
		/// </summary>
		public override void Initialize()
		{
			base.Initialize();
		}

		/// <summary>
		/// nahrání dat
		/// </summary>
		protected override void LoadContent()
		{
			pozadi = hra.Content.Load<Texture2D>(@"Sprity\pozadi_menu");
			base.LoadContent();
		}

		/// <summary>
		/// Real-time logika menu
		/// </summary>
		/// <param name="gameTime">Herní čas</param>
		public override void Update(GameTime gameTime)
		{
			// enter - akce podle vybrané položky
			if (hra.NovaKlavesa(Keys.Enter))
			{
				switch (polozkyMenu.vybranaPolozka.text)
				{
					case "StArT":
						hra.PrepniObrazovku(hra.obrazovkaLevel);
						hra.PrepniHudbu(hra.hudba_zardax);
						break;
					case "SkOrE":
						hra.PrepniObrazovku(hra.obrazovkaSkore);
						break;
					case "AuToRi":
						hra.PrepniObrazovku(hra.obrazovkaAutori);
						break;
					case "KoNeC":
						hra.Exit();
						break;
				}
			}
			base.Update(gameTime);
		}

		/// <summary>
		/// Vykreslení menu
		/// </summary>
		/// <param name="gameTime">Herní čas</param>
		public override void Draw(GameTime gameTime)
		{
			hra.spriteBatch.Begin();
			hra.spriteBatch.Draw(pozadi, new Vector2(0, 0), Color.White);
			hra.spriteBatch.End();
			base.Draw(gameTime);
		}
	}
}
