﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace Robotris
{
	/*
	*  _____ _______         _                      _
	* |_   _|__   __|       | |                    | |
	*   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
	*   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
	*  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
	* |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
	*                   ___
	*                  |  _|___ ___ ___
	*                  |  _|  _| -_| -_|  LICENCE
	*                  |_| |_| |___|___|
	*
	* IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
	*
	* Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
	*
	* Můžete ho upravovat a používat jak chcete, musíte však zmínit
	* odkaz na http://www.itnetwork.cz
	*/

	/// <summary>
	/// This is a game component that implements IUpdateable.
	/// </summary>
	public class KomponentaMraky : Microsoft.Xna.Framework.DrawableGameComponent
	{

		/// <summary>
		/// Změna barvy
		/// </summary>
		private int zmena;
		/// <summary>
		/// Rychlost změny barvy
		/// </summary>
		private int smer;
		/// <summary>
		/// textura mraků
		/// </summary>
		private Texture2D mraky;
		/// <summary>
		/// Pozice mraků
		/// </summary>
		private Vector2 pozice;
		/// <summary>
		/// Instance hry
		/// </summary>
		private Hra hra;

		/// <summary>
		/// Vytvoří novou komponentu KomponentaMraky a předá instanci hry
		/// </summary>
		/// <param name="hra"></param>
		public KomponentaMraky(Hra hra)
			: base(hra)
		{
			this.hra = hra;
		}

		/// <summary>
		/// Inicializace komponenty
		/// </summary>
		public override void Initialize()
		{
			pozice = new Vector2(0, 0);
			zmena = 0;
			smer = 1;

			base.Initialize();
		}

		/// <summary>
		/// Načtení dat
		/// </summary>
		protected override void LoadContent()
		{
			mraky = hra.Content.Load<Texture2D>(@"Sprity\spr_mraky");

			base.LoadContent();
		}

		/// <summary>
		/// Vykreslení
		/// </summary>
		/// <param name="gameTime">Herní čas</param>
		public override void Draw(GameTime gameTime)
		{
			hra.spriteBatch.Begin();
			Color barva = new Color(128 + zmena, 255 - zmena, 128 + zmena);
			for (int i = 0; i < 6; i++)
				hra.spriteBatch.Draw(mraky, new Vector2(pozice.X + i * mraky.Width, 0), barva * 0.8f);
			hra.spriteBatch.End();

			base.Draw(gameTime);
		}

		/// <summary>
		/// Real-time logika
		/// </summary>
		/// <param name="gameTime">Herní čas</param>
		public override void Update(GameTime gameTime)
		{
			// posun mraků doleva
			pozice.X--;
			// návrat na startovní pozici po vyjetí z obrazovky
			if (pozice.X < -(mraky.Width))
				pozice.X = 0;

			// změna barvy mraků dle směru
			zmena += smer;
			if (zmena >= 96)
				smer = -1;
			if (zmena <= 0)
				smer = 1;

			base.Update(gameTime);
		}
	}
}
