﻿using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;


namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	public class KomponentaPolozkyMenu : Microsoft.Xna.Framework.DrawableGameComponent
	{
        /// <summary>
        /// Instance hry
        /// </summary>
		private Hra hra;
        /// <summary>
        /// Seznam položek
        /// </summary>
        private List<PolozkaMenu> polozky;
        /// <summary>
        /// Vybraná položka menu
        /// </summary>
        public PolozkaMenu vybranaPolozka;
        /// <summary>
        /// Pozice vykreslení položek
        /// </summary>
        private Vector2 pozice;
        /// <summary>
        /// Barva nevybrané položky
        /// </summary>
        private Color nevybranaBarva;
        /// <summary>
        /// barva vybrané položky
        /// </summary>
        private Color vybranaBarva;
        /// <summary>
        /// Výška položky
        /// </summary>
        private int vyska;

        /// <summary>
        /// Vytvoří novou komponentu Položky menu
        /// </summary>
        /// <param name="hra">Instance hry</param>
        /// <param name="pozice">Pozice</param>
        /// <param name="nevybranaBarva">Barva nevybrané položky</param>
        /// <param name="vybranaBarva">Barva vybrané položky</param>
        /// <param name="vyska">Výška položky</param>
        public KomponentaPolozkyMenu(Hra hra, Vector2 pozice, Color nevybranaBarva, Color vybranaBarva, int vyska)
            : base(hra)
        {
            this.pozice = pozice;
            this.hra = hra;
            this.nevybranaBarva = nevybranaBarva;
            this.vybranaBarva = vybranaBarva;
            this.vyska = vyska;
            polozky = new List<PolozkaMenu>();
            vybranaPolozka = null;
        }

        /// <summary>
        /// Přidá novou položku v menu
        /// </summary>
        /// <param name="text">Text polo�ky</param>
        public void PridejPolozku(string text)
        {
            // nastavení pozice podle pořadí položky
            Vector2 p = new Vector2(pozice.X, pozice.Y + polozky.Count * vyska);
            PolozkaMenu polozka = new PolozkaMenu(text, p);
            polozky.Add(polozka);
            // výběr první položky
            if (vybranaPolozka == null)
                vybranaPolozka = polozka;
        }

        /// <summary>
        /// Vybere další položku
        /// </summary>
        public void VyberDalsi()
        {
            int index = polozky.IndexOf(vybranaPolozka);
            if (index < polozky.Count - 1)
                vybranaPolozka = polozky[index + 1];
            else
                vybranaPolozka = polozky[0];
        }

        /// <summary>
        /// Vybere předchozí položku
        /// </summary>
        public void VyberPredchozi()
        {
            int index = polozky.IndexOf(vybranaPolozka);
            if (index > 0)
                vybranaPolozka = polozky[index - 1];
            else
                vybranaPolozka = polozky[polozky.Count - 1];
        }

		public override void Initialize()
		{
			base.Initialize();
		}

		protected override void LoadContent()
		{
			base.LoadContent();
		}

		public override void Update(GameTime gameTime)
		{
            // reakce na klávesy
            if (hra.NovaKlavesa(Keys.Up))
                VyberPredchozi();
            if (hra.NovaKlavesa(Keys.Down))
                VyberDalsi();         

			base.Update(gameTime);
		}

		public override void Draw(GameTime gameTime)
		{
            hra.spriteBatch.Begin();
            foreach (PolozkaMenu polozka in polozky)
            {
                Color barva = nevybranaBarva;
                if (polozka == vybranaPolozka)
                    barva = vybranaBarva;
                hra.spriteBatch.TextSeStinemSkalovany(hra.fontBlox, polozka.text, polozka.pozice, barva, polozka.velikost);
            }
            hra.spriteBatch.End();

			base.Draw(gameTime);
		}
	}
}