﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input; // přidáno

namespace Robotris
{
	/*
	*  _____ _______         _                      _
	* |_   _|__   __|       | |                    | |
	*   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
	*   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
	*  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
	* |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
	*                   ___
	*                  |  _|___ ___ ___
	*                  |  _|  _| -_| -_|  LICENCE
	*                  |_| |_| |___|___|
	*
	* IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
	*
	* Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
	*
	* Můžete ho upravovat a používat jak chcete, musíte však zmínit
	* odkaz na http://www.itnetwork.cz
	*/

	/// <summary>
	/// Komponenta pro práci se skóre tabulkou
	/// </summary>
	public class KomponentaSkoreTabulka : DrawableGameComponent
	{
		/// <summary>
		/// Aktuální strana v tabulce
		/// </summary>
		private int strana;
		/// <summary>
		/// List hráčů v tabulce
		/// </summary>
		private List<Hrac> hraci;
		/// <summary>
		/// Instance hry
		/// </summary>
		private Hra hra;
		/// <summary>
		/// Pozadí
		/// </summary>
		private Texture2D pozadi;
		/// <summary>
		/// Zadávaná přezdívka
		/// </summary>
		private string prezdivka;
		/// <summary>
		/// Skóre klient
		/// </summary>
		private SkoreKlient klient;
		/// <summary>
		/// Stav skóre tabulky
		/// </summary>
		private enum eStav
		{
			ChybaNacteni,
			ChybaUlozeni,
			Zapis,
			Vypis,
			Otazka,
		}
		private eStav stav;

		/// <summary>
		/// Vytvoří novou skóre tabulku
		/// </summary>
		public KomponentaSkoreTabulka(Hra hra) : base(hra)
		{
			this.hra = hra;
		}

		/// <summary>
		/// Inicializace komponenty
		/// </summary>
		public override void Initialize()
		{
			hraci = new List<Hrac>();
			strana = 0;
			prezdivka = "";
			klient = new SkoreKlient();

			base.Initialize();
		}

		/// <summary>
		/// Nahrání dat
		/// </summary>
		protected override void LoadContent()
		{
			pozadi = hra.Content.Load<Texture2D>(@"Sprity\pozadi_menu");

			base.LoadContent();
		}

		/// <summary>
		/// Reset stavu při zobrazení komponenty
		/// </summary>
		protected override void OnEnabledChanged(object sender, EventArgs args)
		{
			if (Enabled)
			{
				Nacti();
				// Pokud budeme ve stavu chyba přeskočíme
				if (!(stav == eStav.ChybaNacteni || stav == eStav.ChybaUlozeni))
				{
					// Rovnou na vypis pokud nejsou body
					if (hra.hrac != null && hra.hrac.body > 0)
						stav = eStav.Otazka;
					else
						stav = eStav.Vypis;
				}
			}
			base.OnEnabledChanged(sender, args);
		}

		/// <summary>
		/// Odešle skóre na web
		/// </summary>
		public void Uloz()
		{
			try
			{
				klient.Uloz(hra.hrac);
				Nacti();
				stav = eStav.Vypis;
			}
			catch
			{
				stav = eStav.ChybaUlozeni;
			}
		}

		/// <summary>
		/// Načte skóre
		/// </summary>
		public void Nacti()
		{
			stav = eStav.Vypis;
			try
			{
				hraci = klient.Nacti();
			}
			catch
			{
				stav = eStav.ChybaNacteni;
			}
		}

		/// <summary>
		/// Vykreslení komponenty
		/// </summary>
		/// <param name="gameTime">Herní čas</param>
		public override void Draw(GameTime gameTime)
		{
			hra.spriteBatch.Begin();
			hra.spriteBatch.Draw(pozadi, new Vector2(0, 0), Color.White);

			// výběr vykreslování dle aktuálního stavu
			switch (stav)
			{
				// vykreslení dotazu na nahrání skóre
				case eStav.Otazka:
					hra.spriteBatch.TextSeStinem(hra.fontCourierNew, "Přejete si uožit skóre ?\n\n" +
																	 "[A/N]", new Vector2(510, 240), Color.Red);
					break;
				// vykreslení zadávání přezdívky
				case eStav.Zapis:
					hra.spriteBatch.TextSeStinem(hra.fontCourierNew, "Zadejte jméno:", new Vector2(510, 240), Color.Red);
					hra.spriteBatch.DrawString(hra.fontBlox, prezdivka, new Vector2(510, 280), Color.Yellow);
					break;
				// výpis hráčů v tabulce
				case eStav.Vypis:
					// index posledního hráče v tabulce
					int posledni = (strana * 12) + 11;
					if (posledni > hraci.Count - 1)
						posledni = hraci.Count - 1;
					// index prvního hráče v tabulce
					int prvni = strana * 12;
					// výpis aktuální strany tabulky
					for (int i = prvni; i <= posledni; i++)
					{
						string text = (i + 1).ToString().PadLeft(3) + ". " + hraci[i].prezdivka + ":" + hraci[i].body;
						hra.spriteBatch.TextSeStinem(hra.fontCourierNew, text, new Vector2(510, 240 + (i - prvni) * 28), Color.Red);
					}

					hra.spriteBatch.TextSeStinem(hra.fontCourierNew, "Strana " + (strana + 1), new Vector2(930, 550), Color.Red);
					break;
				// vykreslení chyby načtení
				case eStav.ChybaNacteni:
					hra.spriteBatch.TextSeStinem(hra.fontCourierNew, $"{klient.ChybaNacitani} \n Zkusit znovu? [A/N]", new Vector2(510, 240), Color.Red);
					break;
				// vykreslení chyby uložení
				case eStav.ChybaUlozeni:
					hra.spriteBatch.TextSeStinem(hra.fontCourierNew, $"{klient.ChybaUkladani} \n Zkusit znovu? [A/N]", new Vector2(510, 240), Color.Red);
					break;

			}
			hra.spriteBatch.End();

			base.Draw(gameTime);
		}


		/// <summary>
		/// Real-time logika
		/// </summary>
		/// <param name="gameTime"></param>
		public override void Update(GameTime gameTime)
		{
			// návrat o do menu
			if (hra.klavesy.IsKeyDown(Keys.Escape) == true)
				hra.PrepniObrazovku(hra.obrazovkaMenu);

			// výběr chování dle aktuálního stavu
			switch (stav)
			{
				// dotaz při chybě načtení
				case eStav.ChybaNacteni:
					if (hra.NovaKlavesa(Keys.A)) // ano
						Nacti();
					if (hra.NovaKlavesa(Keys.N)) // ne
						hra.PrepniObrazovku(hra.obrazovkaMenu);
					break;
				// dotaz při chybě uložení
				case eStav.ChybaUlozeni:
					if (hra.NovaKlavesa(Keys.A))
						Uloz();
					if (hra.NovaKlavesa(Keys.N))
						hra.PrepniObrazovku(hra.obrazovkaMenu);
					break;
				// dotaz na nahrání skóre na internet
				case eStav.Otazka:
					if (hra.NovaKlavesa(Keys.A))
						stav = eStav.Zapis;
					if (hra.NovaKlavesa(Keys.N))
						hra.PrepniObrazovku(hra.obrazovkaMenu);
					break;
				// zadávání přezdívky
				case eStav.Zapis:
					// získání všech stisknutých kláves
					Keys[] stisknuteKlavesy = hra.klavesy.GetPressedKeys();
					foreach (var klavesa in stisknuteKlavesy)
					{
						// klávesa je nově stisknuta
						if (hra.klavesyMinule.IsKeyUp(klavesa))
						{
							// mazání backspacem
							if ((klavesa == Keys.Back) && (prezdivka.Length > 0))
								prezdivka = prezdivka.Remove(prezdivka.Length - 1, 1);
							else
							// mezerník
							if (klavesa == Keys.Space)
								prezdivka = prezdivka.Insert(prezdivka.Length, " ");
							else
							// potvrzení enterem
							if (klavesa == Keys.Enter)
							{
								hra.hrac.prezdivka = prezdivka;
								Uloz();
								// vynulujeme uložené body
								hra.hrac.body = 0;
							}
							else
							// osttaní klávesy - vložení písmen
							{
								string pismeno = klavesa.ToString();
								if (pismeno.Length == 2)
									pismeno = pismeno.Replace("D", "");
								pismeno = pismeno.Replace("NumPad", "");
								// přidání písmene do přezdívky
								if ((pismeno.Length == 1) && (prezdivka.Length < 9))
									prezdivka += pismeno;
							}
						}
					}
					break;
				// přepínání stran ve výpisu tabulky
				case eStav.Vypis:
					if (hra.klavesy.IsKeyDown(Keys.Up) && hra.klavesyMinule.IsKeyUp(Keys.Up))
						if (strana > 0)
							strana--;
					if (hra.klavesy.IsKeyDown(Keys.Down) && hra.klavesyMinule.IsKeyUp(Keys.Down))
						if (strana < (int)Math.Ceiling((double)hraci.Count / 12) - 1)
							strana++;
					break;
			}

			base.Update(gameTime);
		}
	}
}
