﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Linq;

namespace Robotris
{
	/*
	*  _____ _______         _                      _
	* |_   _|__   __|       | |                    | |
	*   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
	*   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
	*  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
	* |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
	*                   ___
	*                  |  _|___ ___ ___
	*                  |  _|  _| -_| -_|  LICENCE
	*                  |_| |_| |___|___|
	*
	* IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
	*
	* Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
	*
	* Můžete ho upravovat a používat jak chcete, musíte však zmínit
	* odkaz na http://www.itnetwork.cz
	*/

	/// <summary>
	/// Třída se stará o ukládání a nahrávání skóre
	/// </summary>
	public class SkoreKlient
	{
		/// <summary>
		/// Hláška pokud se nepovedlo načtení
		/// </summary>
		public string ChybaNacitani => $"Skóre se nepodařilo načíst.\n Soubor '{soubor}' pravděpodobně neexistuje";
		/// <summary>
		/// Hláška pokud se nepovedlo uložení
		/// </summary>
		public string ChybaUkladani => $"Skóre se nepodařilo uložit.\n Vyskytla se chyba při ukládání souboru.";

		/// <summary>
		/// Cesta k souboru se skóresss
		/// </summary>
		private readonly string soubor = "skore.xml";

		/// <summary>
		/// Uloží instanci hráče do skóre
		/// </summary>
		/// <param name="hrac"></param>
		public void Uloz(Hrac hrac)
		{
			// Pokud soubor neexistuje
			if (!File.Exists(soubor))
			{
				// nastavení xml zapisovače
				XmlWriterSettings xmlWriterSettings = new XmlWriterSettings();
				xmlWriterSettings.Indent = true;
				xmlWriterSettings.NewLineOnAttributes = true;
				// vytvoření xml zapisovače
				using (XmlWriter xmlWriter = XmlWriter.Create(soubor, xmlWriterSettings))
				{
					// xml hlavička
					xmlWriter.WriteStartDocument();
					// xml kořenový element
					xmlWriter.WriteStartElement("hraci");

					// xml element s hráčem
					xmlWriter.WriteStartElement("hrac");
					// přidání přezdívky do elementu hráče
					xmlWriter.WriteElementString("prezdivka", hrac.prezdivka);
					// přidání bodů do elementu hráče
					xmlWriter.WriteElementString("body", hrac.body.ToString());
					// zavření elementu hráče
					xmlWriter.WriteEndElement();

					// zavření elementu hráči a celého dokumentu
					xmlWriter.WriteEndDocument();
					// vyčištění paměti
					xmlWriter.Flush();
					// zavření zapisovače
					xmlWriter.Close();
				}
			}
			else
			{
				// načtení xml dokumentu
				XDocument xDocument = XDocument.Load(soubor);
				// získání kořenoého elementu hráči
				XElement koren = xDocument.Element("hraci");
				// získání všech zapsaných hráču
				IEnumerable<XElement> polozky = koren.Descendants("hrac");
				// vybrání prvního (posledního zapsaného)
				XElement polozka = polozky.First();
				// zapsat před prvního (posledního zapsaného)
				polozka.AddBeforeSelf(
				   new XElement("hrac",
					   new XElement("prezdivka", hrac.prezdivka),
					   new XElement("body", hrac.body)));
				// uložit celý dokument s aktuální změnou
				xDocument.Save(soubor);
			}
		}

		/// <summary>
		/// Načte skóre tabulku z XML do listu hráčů
		/// </summary>
		/// <returns>List hráčů</returns>
		public List<Hrac> Nacti()
		{
			string element = "";
			var hrac = new Hrac();
			var hraci = new List<Hrac>();

			// parsování XML
			using (var xr = XmlReader.Create(soubor))
			{
				// postupné načítání uzlů
				while (xr.Read())
				{
					// uzel typu element
					if (xr.NodeType == XmlNodeType.Element)
						element = xr.Name;  // uložení jména elementu
											// uzel typu text
					else
					if (xr.NodeType == XmlNodeType.Text)
					{
						switch (element)
						{
							case "prezdivka":
								hrac.prezdivka = xr.Value;
								break;
							case "body":
								hrac.body = long.Parse(xr.Value);
								break;
						}
					}
					// konec elementu hráč
					else if ((xr.NodeType == XmlNodeType.EndElement) && (xr.Name == "hrac"))
					{
						hraci.Add(hrac);
						hrac = new Hrac();
					}
				}
			}
			return hraci;
		}
	}
}
